// The contents of this file will be executed before any of
// your view controllers are ever executed, including the index.
// You have access to all functionality on the `Alloy` namespace.
//
// This is a great place to do any initialization for your app
// or create any global variables/functions that you'd like to
// make available throughout your app. You can easily make things
// accessible globally by attaching them to the `Alloy.Globals`
// object. For example:
//
// Alloy.Globals.someGlobalFunction = function(){};

// It would be clearer if we used moment for all date/time calculations and processing
var moment = require('alloy/moment');

(function() {
	Alloy.Globals.NotificationsEnum = {
		NONE: "none",
		DAILY: "daily",
		WEEKLY: "weekly"
	};

	Alloy.Globals.day = Ti.App.Properties.getString('day', 'Sunday'); // default to sunday
	Alloy.Globals.time = new Date(Ti.App.Properties.getString('time', 'Sep 27 1998 19:00:00')); // default to 7:00pm notifications

	function setTimeOnDate(date, time) {
		date.setHours(time.getHours());
		date.setMinutes(time.getMinutes());
		date.setSeconds(0);
		date.setMilliseconds(0);
	}

	function adjustActivityReminderIfNecessary(currentDate, frequency)
	{
		// Fetch the settings model data for display
		var mySettings = Alloy.Models.instance('settings');
		mySettings.fetch();
		
		var now = moment();
		var dates = mySettings.get('dates');
		var startEndDateSplits = dates.split('-');
		var challengeStartDate = moment(startEndDateSplits[0] + " 00:00:00");
		var challengeEndDate = moment(startEndDateSplits[1] + " 23:59:59");
		
		
		if (currentDate > challengeEndDate) {
			// If this notification occurs after the challenge date range don't schedule it, by returning null.
			console.log('Next activity reminder date ' + currentDate + ' is after: ' + challengeEndDate.format());
			currentDate = null;
		} else {
			// If this notification occurs before the challenge start date or in the past schedule it at next logical time in future during the challenge.
			console.log('Next activity reminder date ' + currentDate + ' is before: ' + challengeEndDate.format());
			var daysIncrement = (frequency == 'daily') ? 1 : 7;
			
			var nextPossibleDate = moment(currentDate);
			if (nextPossibleDate < now) {
				nextPossibleDate.add(daysIncrement, 'days');
			}
			var earliestPossibleDate = challengeStartDate > nextPossibleDate ? challengeStartDate : nextPossibleDate;
			while (nextPossibleDate < earliestPossibleDate) {
				nextPossibleDate.add(daysIncrement, 'days');
				console.log('Bump activity reminder date up to: ' + nextPossibleDate.toDate());
			}
			currentDate = nextPossibleDate.toDate();
			console.log('Activity reminder finally set at: ' + currentDate);
		}
		
		return currentDate;
	}

	function nextDay() {
		// TODO: redo using moment.js if it would be better
		var currentDate = new Date();
		setTimeOnDate(currentDate, Alloy.Globals.time);
		currentDate = adjustActivityReminderIfNecessary(currentDate, 'daily');
		return currentDate;
	}

	// if the specified time for the notification hasn't happened yet this week, do the notification this week
	function nextWeek() {
		// TODO: redo using moment.js if it would be better
		
		// get next monday
		var currentDate = new Date();
		var currentDay = currentDate.getDay() - 1; // monday should be zero
		var dayInt;
		switch (Alloy.Globals.day) {
			case "Monday":
				dayInt = 0;
				break;
			case "Tuesday":
				dayInt = 1;
				break;
			case "Wednesday":
				dayInt = 2;
				break;
			case "Thursday":
				dayInt = 3;
				break;
			case "Friday":
				dayInt = 4;
				break;
			case "Saturday":
				dayInt = 5;
				break;
			case "Sunday":
				dayInt = 6;
				break;
		}
		
		if (dayInt < currentDay) {
			var DAYS_TO_TIME = 24 * 60 * 60 * 1000;
			var nextMonday = new Date(currentDate.getTime() + (7 - currentDay) * DAYS_TO_TIME);
			notifyDate = new Date(nextMonday.getTime() + dayInt * DAYS_TO_TIME);
		} else if (dayInt == currentDay) {
			notifyDate = currentDate;
		} else {
			var DAYS_TO_TIME = 24 * 60 * 60 * 1000;
			notifyDate = new Date(currentDate.getTime() + (dayInt - currentDay) * DAYS_TO_TIME);
		}
		setTimeOnDate(notifyDate, Alloy.Globals.time);

		notifyDate = adjustActivityReminderIfNecessary(notifyDate, 'weekly');

		return notifyDate;
	}

	function dailyNotificationIOS() {
		var date = nextDay();
		
		if (date) {
			return {
				alertAction: "record activities",
				alertBody: "Have you recorded your healthy activities for today?",
				date: date,
				repeat: "daily",
				sound: "default",
				badge: -1,
				userInfo: {
					"id": "hic_notify"
				}
			};
		} else {
				return null;
		}
	}

	function weeklyNotificationIOS() {
		var date = nextDay();
		
		if (date) {
			return {
				alertAction: "record activities",
				alertBody: "Have you recorded your healthy activities for this week?",
				date: nextWeek(),
				repeat: "weekly",
				sound: "default",
				badge: -1,
				userInfo: {
					"id": "hic_notify"
				}
			};
		} else {
			return null;
		}
	}

	function dailyNotificationAndroid() {
		return nextDay();
	}

	function weeklyNotificationAndroid() {
		return nextWeek();
	}
	
	var alarmManager;
	if (OS_ANDROID) {
		var alarmModule = require('bencoding.alarmmanager');
		alarmManager = alarmModule.createAlarmManager();

		var isRunning = Ti.App.Properties.getBool("service_running", false);//get service running bool status
		if (isRunning) {
		    Ti.API.info('service is running');
		} else {
		    Ti.API.info('service is not running');
		}
	}

	// Check if the device is running iOS 8 or later, before registering for local notifications
	if (Ti.Platform.name == "iPhone OS" && parseInt(Ti.Platform.version.split(".")[0]) >= 8) {
	    Ti.App.iOS.registerUserNotificationSettings({
		    types: [
	            Ti.App.iOS.USER_NOTIFICATION_TYPE_ALERT,
	            Ti.App.iOS.USER_NOTIFICATION_TYPE_SOUND,
	            Ti.App.iOS.USER_NOTIFICATION_TYPE_BADGE
	        ]
	    });
	}

	// Function changes the notifications settings
	Alloy.Globals.changeNotificationsSettings = function (frequency) {
		Ti.App.Properties.setString('notifications', frequency);
		if (OS_IOS) {
			// Remove previous notifications
			Ti.App.iOS.cancelLocalNotification("hic_notify");
			// Add new activity notifications
			var notification = null;
			switch (frequency) {
				case Alloy.Globals.NotificationsEnum.NONE:
					// do nothing because there are no notifications to add
					break;
				case Alloy.Globals.NotificationsEnum.DAILY:
					notification = dailyNotificationIOS();
					break;
				case Alloy.Globals.NotificationsEnum.WEEKLY:
					notification = weeklyNotificationIOS();
					break;
			}
			if (notification) {
				console.log("Registered " + (frequency == Alloy.Globals.NotificationsEnum.DAILY ? "daily" : "weekly") + " notifications for: ");
				console.log(notification);
				console.log("The current time is: ");
				console.log(new Date());
				Ti.App.iOS.scheduleLocalNotification(notification);
			}
		}
		if (OS_ANDROID) {
			alarmManager.cancelAlarmService(1); 

			var date, repeatInterval;
			switch (frequency) {
				case Alloy.Globals.NotificationsEnum.NONE:
					// return because there is no notification to schedule
					return;
					break;
				case Alloy.Globals.NotificationsEnum.DAILY:
					console.log("Registered daily notifications for: ");
					date = dailyNotificationAndroid();
					repeatInterval = "daily";
					console.log(date);
					console.log("The current time is: ");
					console.log(new Date());
					break;
				case Alloy.Globals.NotificationsEnum.WEEKLY:
					console.log("Registered weekly notifications for: ");
					date = weeklyNotificationAndroid();
					repeatInterval = "weekly";
					console.log(date);
					console.log("The current time is: ");
					console.log(new Date());
					break;
			}
			
			if (date) {
				// Send the notification.
				alarmManager.addAlarmService({
		        	service : 'edu.jhuapl.InTheZoneChallengeTracker.DailyEventNotificationService', // your service name as in AndroidManifest.xml (DJL: Not sure how important this is as I don't see anything using it elsewhere)
		        	requesCode: 1,
		        	year: date.getFullYear(),
					month: date.getMonth(),
					day: date.getDate(),
					hour: date.getHours(),
					minute: date.getMinutes(),
		        	repeat : repeatInterval,
		        	customData: JSON.stringify(['Have you recorded your healthy activities recently?', 'Tap to record them']) // TODO: Restructure Android notifications to be like iOS in code so we can specify daily or weekly in string.
		    	});
		   }
		}
	};

	var loaded = false;

	Alloy.Globals.isLoaded = function () {
		return loaded;
	};

	var callbacks = [];

	// Loading callback system
	Alloy.Globals.registerLoadingCallback = function (cb) {
		callbacks.push(cb);
	};

	Alloy.Globals.doneLoading = function () {
		var i;
		loaded = true;
		for (i = 0; i < callbacks.length; i++) {
			callbacks[i]();
		}
	};
})();

// These lines are commented out because we are not using ACS

// // added during app creation. this will automatically login to
// // ACS for your application and then fire an event (see below)
// // when connected or errored. if you do not use ACS in your
// // application as a client, you should remove this block
// (function(){
// var ACS = require('ti.cloud'),
//     env = Ti.App.deployType.toLowerCase() === 'production' ? 'production' : 'development',
//     username = Ti.App.Properties.getString('acs-username-'+env),
//     password = Ti.App.Properties.getString('acs-password-'+env);
//
// // if not configured, just return
// if (!env || !username || !password) { return; }
// /**
//  * Appcelerator Cloud (ACS) Admin User Login Logic
//  *
//  * fires login.success with the user as argument on success
//  * fires login.failed with the result as argument on error
//  */
// ACS.Users.login({
// 	login:username,
// 	password:password,
// }, function(result){
// 	if (env==='development') {
// 		Ti.API.info('ACS Login Results for environment `'+env+'`:');
// 		Ti.API.info(result);
// 	}
// 	if (result && result.success && result.users && result.users.length){
// 		Ti.App.fireEvent('login.success',result.users[0],env);
// 	} else {
// 		Ti.App.fireEvent('login.failed',result,env);
// 	}
// });
//
// })();
